@@ -6,10 +6,12 @@ import re
 import shutil
 import subprocess
 import sys
+import time
 from contextlib import contextmanager
 
 import pytest
 import six
+from mock import patch
 from pip._vendor.contextlib2 import ExitStack, nullcontext
 from setuptools.wheel import Wheel
 
@@ -25,7 +27,8 @@ from tests.lib.venv import VirtualEnvironment
 if MYPY_CHECK_RUNNING:
     from typing import Dict, Iterable
 
-    from tests.lib.server import MockServer as _MockServer, Responder
+    from tests.lib.server import MockServer as _MockServer
+    from tests.lib.server import Responder
 
 
 def pytest_addoption(parser):
@@ -36,16 +39,11 @@ def pytest_addoption(parser):
         help="keep temporary test directories",
     )
     parser.addoption(
-        "--new-resolver",
-        action="store_true",
-        default=False,
-        help="use new resolver in tests",
-    )
-    parser.addoption(
-        "--new-resolver-runtests",
-        action="store_true",
-        default=False,
-        help="run the skipped tests for the new resolver",
+        "--resolver",
+        action="store",
+        default="2020-resolver",
+        choices=["2020-resolver", "legacy"],
+        help="use given resolver in tests",
     )
     parser.addoption(
         "--use-venv",
@@ -60,16 +58,10 @@ def pytest_collection_modifyitems(config, items):
         if not hasattr(item, 'module'):  # e.g.: DoctestTextfile
             continue
 
-        # Mark network tests as flaky
-        if (item.get_closest_marker('network') is not None and
-                "CI" in os.environ):
-            item.add_marker(pytest.mark.flaky(reruns=3))
-
-        if (item.get_closest_marker('fails_on_new_resolver') and
-                config.getoption("--new-resolver") and
-                not config.getoption("--new-resolver-runtests")):
-            item.add_marker(pytest.mark.skip(
-                'This test does not work with the new resolver'))
+        if "CI" in os.environ:
+            # Mark network tests as flaky
+            if item.get_closest_marker('network') is not None:
+                item.add_marker(pytest.mark.flaky(reruns=3, reruns_delay=2))
 
         if six.PY3:
             if (item.get_closest_marker('incompatible_with_test_venv') and
@@ -100,15 +92,32 @@ def pytest_collection_modifyitems(config, items):
 
 
 @pytest.fixture(scope="session", autouse=True)
-def use_new_resolver(request):
-    """Set environment variable to make pip default to the new resolver.
+def resolver_variant(request):
+    """Set environment variable to make pip default to the correct resolver.
     """
-    new_resolver = request.config.getoption("--new-resolver")
-    if new_resolver:
-        os.environ["PIP_USE_FEATURE"] = "2020-resolver"
+    resolver = request.config.getoption("--resolver")
+
+    # Handle the environment variables for this test.
+    features = set(os.environ.get("PIP_USE_FEATURE", "").split())
+    deprecated_features = set(os.environ.get("PIP_USE_DEPRECATED", "").split())
+
+    if six.PY3:
+        if resolver == "legacy":
+            deprecated_features.add("legacy-resolver")
+        else:
+            deprecated_features.discard("legacy-resolver")
     else:
-        os.environ.pop("PIP_USE_FEATURE", None)
-    yield new_resolver
+        if resolver == "2020-resolver":
+            features.add("2020-resolver")
+        else:
+            features.discard("2020-resolver")
+
+    env = {
+        "PIP_USE_FEATURE": " ".join(features),
+        "PIP_USE_DEPRECATED": " ".join(deprecated_features),
+    }
+    with patch.dict(os.environ, env):
+        yield resolver
 
 
 @pytest.fixture(scope='session')
@@ -151,7 +160,7 @@ def tmpdir(request, tmpdir):
 
 
 @pytest.fixture(autouse=True)
-def isolate(tmpdir):
+def isolate(tmpdir, monkeypatch):
     """
     Isolate our tests so that things like global configuration files and the
     like do not affect our test results.
@@ -174,45 +183,51 @@ def isolate(tmpdir):
     if sys.platform == 'win32':
         # Note: this will only take effect in subprocesses...
         home_drive, home_path = os.path.splitdrive(home_dir)
-        os.environ.update({
-            'USERPROFILE': home_dir,
-            'HOMEDRIVE': home_drive,
-            'HOMEPATH': home_path,
-        })
+        monkeypatch.setenv('USERPROFILE', home_dir)
+        monkeypatch.setenv('HOMEDRIVE', home_drive)
+        monkeypatch.setenv('HOMEPATH', home_path)
         for env_var, sub_path in (
             ('APPDATA', 'AppData/Roaming'),
             ('LOCALAPPDATA', 'AppData/Local'),
         ):
             path = os.path.join(home_dir, *sub_path.split('/'))
-            os.environ[env_var] = path
+            monkeypatch.setenv(env_var, path)
             os.makedirs(path)
     else:
         # Set our home directory to our temporary directory, this should force
         # all of our relative configuration files to be read from here instead
         # of the user's actual $HOME directory.
-        os.environ["HOME"] = home_dir
+        monkeypatch.setenv("HOME", home_dir)
         # Isolate ourselves from XDG directories
-        os.environ["XDG_DATA_HOME"] = os.path.join(home_dir, ".local", "share")
-        os.environ["XDG_CONFIG_HOME"] = os.path.join(home_dir, ".config")
-        os.environ["XDG_CACHE_HOME"] = os.path.join(home_dir, ".cache")
-        os.environ["XDG_RUNTIME_DIR"] = os.path.join(home_dir, ".runtime")
-        os.environ["XDG_DATA_DIRS"] = ":".join([
+        monkeypatch.setenv("XDG_DATA_HOME", os.path.join(
+            home_dir, ".local", "share",
+        ))
+        monkeypatch.setenv("XDG_CONFIG_HOME", os.path.join(
+            home_dir, ".config",
+        ))
+        monkeypatch.setenv("XDG_CACHE_HOME", os.path.join(home_dir, ".cache"))
+        monkeypatch.setenv("XDG_RUNTIME_DIR", os.path.join(
+            home_dir, ".runtime",
+        ))
+        monkeypatch.setenv("XDG_DATA_DIRS", os.pathsep.join([
             os.path.join(fake_root, "usr", "local", "share"),
             os.path.join(fake_root, "usr", "share"),
-        ])
-        os.environ["XDG_CONFIG_DIRS"] = os.path.join(fake_root, "etc", "xdg")
+        ]))
+        monkeypatch.setenv("XDG_CONFIG_DIRS", os.path.join(
+            fake_root, "etc", "xdg",
+        ))
 
     # Configure git, because without an author name/email git will complain
     # and cause test failures.
-    os.environ["GIT_CONFIG_NOSYSTEM"] = "1"
-    os.environ["GIT_AUTHOR_NAME"] = "pip"
-    os.environ["GIT_AUTHOR_EMAIL"] = "distutils-sig@python.org"
+    monkeypatch.setenv("GIT_CONFIG_NOSYSTEM", "1")
+    monkeypatch.setenv("GIT_AUTHOR_NAME", "pip")
+    monkeypatch.setenv("GIT_AUTHOR_EMAIL", "distutils-sig@python.org")
 
     # We want to disable the version check from running in the tests
-    os.environ["PIP_DISABLE_PIP_VERSION_CHECK"] = "true"
+    monkeypatch.setenv("PIP_DISABLE_PIP_VERSION_CHECK", "true")
 
     # Make sure tests don't share a requirements tracker.
-    os.environ.pop('PIP_REQ_TRACKER', None)
+    monkeypatch.delenv("PIP_REQ_TRACKER", False)
 
     # FIXME: Windows...
     os.makedirs(os.path.join(home_dir, ".config", "git"))
@@ -470,8 +485,8 @@ def in_memory_pip():
 
 @pytest.fixture(scope="session")
 def deprecated_python():
-    """Used to indicate whether pip deprecated this python version"""
-    return sys.version_info[:2] in [(2, 7)]
+    """Used to indicate whether pip deprecated this Python version"""
+    return sys.version_info[:2] in [(2, 7), (3, 5)]
 
 
 @pytest.fixture(scope="session")
@@ -547,3 +562,13 @@ def mock_server():
     test_server = MockServer(server)
     with test_server.context:
         yield test_server
+
+
+@pytest.fixture
+def utc():
+    # time.tzset() is not implemented on some platforms, e.g. Windows.
+    tzset = getattr(time, 'tzset', lambda: None)
+    with patch.dict(os.environ, {'TZ': 'UTC'}):
+        tzset()
+        yield
+    tzset()
