@@ -1,3 +1,4 @@
+import contextlib
 import os
 import shutil
 import sys
@@ -17,7 +18,6 @@ from pip._internal.exceptions import (
     InvalidWheelFilename,
     PreviousBuildDirError,
 )
-from pip._internal.legacy_resolve import Resolver
 from pip._internal.network.session import PipSession
 from pip._internal.operations.prepare import RequirementPreparer
 from pip._internal.req import InstallRequirement, RequirementSet
@@ -26,23 +26,35 @@ from pip._internal.req.constructors import (
     _looks_like_path,
     install_req_from_editable,
     install_req_from_line,
+    install_req_from_parsed_requirement,
     install_req_from_req_string,
     parse_editable,
 )
-from pip._internal.req.req_file import process_line
-from pip._internal.req.req_tracker import RequirementTracker
-from pip._internal.utils.urls import path_to_url
-from tests.lib import (
-    DATA_DIR,
-    assert_raises_regexp,
-    make_test_finder,
-    requirements_file,
+from pip._internal.req.req_file import (
+    ParsedLine,
+    get_line_parser,
+    handle_requirement_line,
 )
+from pip._internal.req.req_tracker import get_requirement_tracker
+from pip._internal.resolution.legacy.resolver import Resolver
+from pip._internal.utils.urls import path_to_url
+from tests.lib import assert_raises_regexp, make_test_finder, requirements_file
 
 
 def get_processed_req_from_line(line, fname='file', lineno=1):
-    req = list(process_line(line, fname, lineno))[0]
-    req.is_direct = True
+    line_parser = get_line_parser(None)
+    args_str, opts = line_parser(line)
+    parsed_line = ParsedLine(
+        fname,
+        lineno,
+        args_str,
+        opts,
+        False,
+    )
+    parsed_req = handle_requirement_line(parsed_line)
+    assert parsed_req is not None
+    req = install_req_from_parsed_requirement(parsed_req)
+    req.user_supplied = True
     return req
 
 
@@ -55,30 +67,38 @@ class TestRequirementSet(object):
     def teardown(self):
         shutil.rmtree(self.tempdir, ignore_errors=True)
 
-    def _basic_resolver(self, finder):
-        preparer = RequirementPreparer(
-            build_dir=os.path.join(self.tempdir, 'build'),
-            src_dir=os.path.join(self.tempdir, 'src'),
-            download_dir=None,
-            wheel_download_dir=None,
-            progress_bar="on",
-            build_isolation=True,
-            req_tracker=RequirementTracker(),
-        )
+    @contextlib.contextmanager
+    def _basic_resolver(self, finder, require_hashes=False):
         make_install_req = partial(
             install_req_from_req_string,
             isolated=False,
-            wheel_cache=None,
             use_pep517=None,
         )
-        return Resolver(
-            preparer=preparer,
-            make_install_req=make_install_req,
-            session=PipSession(), finder=finder,
-            use_user_site=False, upgrade_strategy="to-satisfy-only",
-            ignore_dependencies=False, ignore_installed=False,
-            ignore_requires_python=False, force_reinstall=False,
-        )
+        session = PipSession()
+
+        with get_requirement_tracker() as tracker:
+            preparer = RequirementPreparer(
+                build_dir=os.path.join(self.tempdir, 'build'),
+                src_dir=os.path.join(self.tempdir, 'src'),
+                download_dir=None,
+                build_isolation=True,
+                req_tracker=tracker,
+                session=session,
+                progress_bar='on',
+                finder=finder,
+                require_hashes=require_hashes,
+                use_user_site=False,
+                lazy_wheel=False,
+            )
+            yield Resolver(
+                preparer=preparer,
+                make_install_req=make_install_req,
+                finder=finder,
+                wheel_cache=None,
+                use_user_site=False, upgrade_strategy="to-satisfy-only",
+                ignore_dependencies=False, ignore_installed=False,
+                ignore_requires_python=False, force_reinstall=False,
+            )
 
     def test_no_reuse_existing_build_dir(self, data):
         """Test prepare_files raise exception with previous build dir"""
@@ -89,17 +109,19 @@ class TestRequirementSet(object):
             pass
         reqset = RequirementSet()
         req = install_req_from_line('simple')
-        req.is_direct = True
+        req.user_supplied = True
         reqset.add_requirement(req)
         finder = make_test_finder(find_links=[data.find_links])
-        resolver = self._basic_resolver(finder)
-        assert_raises_regexp(
-            PreviousBuildDirError,
-            r"pip can't proceed with [\s\S]*%s[\s\S]*%s" %
-            (req, build_dir.replace('\\', '\\\\')),
-            resolver.resolve,
-            reqset,
-        )
+        with self._basic_resolver(finder) as resolver:
+            assert_raises_regexp(
+                PreviousBuildDirError,
+                r"pip can't proceed with [\s\S]*{req}[\s\S]*{build_dir_esc}"
+                .format(
+                    build_dir_esc=build_dir.replace('\\', '\\\\'), req=req),
+                resolver.resolve,
+                reqset.all_requirements,
+                True,
+            )
 
     # TODO: Update test when Python 2.7 is dropped.
     def test_environment_marker_extras(self, data):
@@ -111,98 +133,51 @@ class TestRequirementSet(object):
         req = install_req_from_editable(
             data.packages.joinpath("LocalEnvironMarker")
         )
-        req.is_direct = True
+        req.user_supplied = True
         reqset.add_requirement(req)
         finder = make_test_finder(find_links=[data.find_links])
-        resolver = self._basic_resolver(finder)
-        resolver.resolve(reqset)
+        with self._basic_resolver(finder) as resolver:
+            reqset = resolver.resolve(reqset.all_requirements, True)
         # This is hacky but does test both case in py2 and py3
         if sys.version_info[:2] == (2, 7):
             assert reqset.has_requirement('simple')
         else:
             assert not reqset.has_requirement('simple')
 
-    @pytest.mark.network
-    def test_missing_hash_checking(self):
-        """Make sure prepare_files() raises an error when a requirement has no
-        hash in implicit hash-checking mode.
-        """
-        reqset = RequirementSet()
-        # No flags here. This tests that detection of later flags nonetheless
-        # requires earlier packages to have hashes:
-        reqset.add_requirement(get_processed_req_from_line(
-            'blessings==1.0', lineno=1
-        ))
-        # This flag activates --require-hashes mode:
-        reqset.add_requirement(get_processed_req_from_line(
-            'tracefront==0.1 --hash=sha256:somehash', lineno=2,
-        ))
-        # This hash should be accepted because it came from the reqs file, not
-        # from the internet:
-        reqset.add_requirement(get_processed_req_from_line(
-            'https://files.pythonhosted.org/packages/source/m/more-itertools/'
-            'more-itertools-1.0.tar.gz#md5=b21850c3cfa7efbb70fd662ab5413bdd',
-            lineno=3,
-        ))
-        # The error text should list this as a URL and not `peep==3.1.1`:
-        reqset.add_requirement(get_processed_req_from_line(
-            'https://files.pythonhosted.org/'
-            'packages/source/p/peep/peep-3.1.1.tar.gz',
-            lineno=4,
-        ))
-        finder = make_test_finder(index_urls=['https://pypi.org/simple/'])
-        resolver = self._basic_resolver(finder)
-        assert_raises_regexp(
-            HashErrors,
-            r'Hashes are required in --require-hashes mode, but they are '
-            r'missing .*\n'
-            r'    https://files\.pythonhosted\.org/packages/source/p/peep/peep'
-            r'-3\.1\.1\.tar\.gz --hash=sha256:[0-9a-f]+\n'
-            r'    blessings==1.0 --hash=sha256:[0-9a-f]+\n'
-            r'THESE PACKAGES DO NOT MATCH THE HASHES.*\n'
-            r'    tracefront==0.1 .*:\n'
-            r'        Expected sha256 somehash\n'
-            r'             Got        [0-9a-f]+$',
-            resolver.resolve,
-            reqset
-        )
-
     def test_missing_hash_with_require_hashes(self, data):
         """Setting --require-hashes explicitly should raise errors if hashes
         are missing.
         """
-        reqset = RequirementSet(require_hashes=True)
+        reqset = RequirementSet()
         reqset.add_requirement(get_processed_req_from_line(
             'simple==1.0', lineno=1
         ))
 
         finder = make_test_finder(find_links=[data.find_links])
-        resolver = self._basic_resolver(finder)
 
-        assert_raises_regexp(
-            HashErrors,
-            r'Hashes are required in --require-hashes mode, but they are '
-            r'missing .*\n'
-            r'    simple==1.0 --hash=sha256:393043e672415891885c9a2a0929b1af95'
-            r'fb866d6ca016b42d2e6ce53619b653$',
-            resolver.resolve,
-            reqset
-        )
+        with self._basic_resolver(finder, require_hashes=True) as resolver:
+            assert_raises_regexp(
+                HashErrors,
+                r'Hashes are required in --require-hashes mode, but they are '
+                r'missing .*\n'
+                r'    simple==1.0 --hash=sha256:393043e672415891885c9a2a0929b1'
+                r'af95fb866d6ca016b42d2e6ce53619b653$',
+                resolver.resolve,
+                reqset.all_requirements,
+                True,
+            )
 
     def test_missing_hash_with_require_hashes_in_reqs_file(self, data, tmpdir):
         """--require-hashes in a requirements file should make its way to the
         RequirementSet.
         """
-        req_set = RequirementSet(require_hashes=False)
         finder = make_test_finder(find_links=[data.find_links])
         session = finder._link_collector.session
         command = create_command('install')
         with requirements_file('--require-hashes', tmpdir) as reqs_file:
             options, args = command.parse_args(['-r', reqs_file])
-            command.populate_requirement_set(
-                req_set, args, options, finder, session, wheel_cache=None,
-            )
-        assert req_set.require_hashes
+            command.get_requirements(args, options, finder, session)
+        assert options.require_hashes
 
     def test_unsupported_hashes(self, data):
         """VCS and dir links should raise errors when --require-hashes is
@@ -212,33 +187,37 @@ class TestRequirementSet(object):
         should trump the presence or absence of a hash.
 
         """
-        reqset = RequirementSet(require_hashes=True)
+        reqset = RequirementSet()
         reqset.add_requirement(get_processed_req_from_line(
             'git+git://github.com/pypa/pip-test-package --hash=sha256:123',
             lineno=1,
         ))
         dir_path = data.packages.joinpath('FSPkg')
         reqset.add_requirement(get_processed_req_from_line(
-            'file://%s' % (dir_path,),
+            'file://{dir_path}'.format(**locals()),
             lineno=2,
         ))
         finder = make_test_finder(find_links=[data.find_links])
-        resolver = self._basic_resolver(finder)
+
         sep = os.path.sep
         if sep == '\\':
             sep = '\\\\'  # This needs to be escaped for the regex
-        assert_raises_regexp(
-            HashErrors,
-            r"Can't verify hashes for these requirements because we don't "
-            r"have a way to hash version control repositories:\n"
-            r"    git\+git://github\.com/pypa/pip-test-package \(from -r file "
-            r"\(line 1\)\)\n"
-            r"Can't verify hashes for these file:// requirements because they "
-            r"point to directories:\n"
-            r"    file://.*{sep}data{sep}packages{sep}FSPkg "
-            r"\(from -r file \(line 2\)\)".format(sep=sep),
-            resolver.resolve,
-            reqset)
+
+        with self._basic_resolver(finder, require_hashes=True) as resolver:
+            assert_raises_regexp(
+                HashErrors,
+                r"Can't verify hashes for these requirements because we don't "
+                r"have a way to hash version control repositories:\n"
+                r"    git\+git://github\.com/pypa/pip-test-package \(from -r "
+                r"file \(line 1\)\)\n"
+                r"Can't verify hashes for these file:// requirements because "
+                r"they point to directories:\n"
+                r"    file://.*{sep}data{sep}packages{sep}FSPkg "
+                r"\(from -r file \(line 2\)\)".format(sep=sep),
+                resolver.resolve,
+                reqset.all_requirements,
+                True,
+            )
 
     def test_unpinned_hash_checking(self, data):
         """Make sure prepare_files() raises an error when a requirement is not
@@ -257,55 +236,62 @@ class TestRequirementSet(object):
             lineno=2,
         ))
         finder = make_test_finder(find_links=[data.find_links])
-        resolver = self._basic_resolver(finder)
-        assert_raises_regexp(
-            HashErrors,
-            # Make sure all failing requirements are listed:
-            r'versions pinned with ==. These do not:\n'
-            r'    simple .* \(from -r file \(line 1\)\)\n'
-            r'    simple2>1.0 .* \(from -r file \(line 2\)\)',
-            resolver.resolve,
-            reqset)
+        with self._basic_resolver(finder, require_hashes=True) as resolver:
+            assert_raises_regexp(
+                HashErrors,
+                # Make sure all failing requirements are listed:
+                r'versions pinned with ==. These do not:\n'
+                r'    simple .* \(from -r file \(line 1\)\)\n'
+                r'    simple2>1.0 .* \(from -r file \(line 2\)\)',
+                resolver.resolve,
+                reqset.all_requirements,
+                True,
+            )
 
     def test_hash_mismatch(self, data):
         """A hash mismatch should raise an error."""
         file_url = path_to_url(
             (data.packages / 'simple-1.0.tar.gz').resolve())
-        reqset = RequirementSet(require_hashes=True)
+        reqset = RequirementSet()
         reqset.add_requirement(get_processed_req_from_line(
-            '%s --hash=sha256:badbad' % file_url, lineno=1,
+            '{file_url} --hash=sha256:badbad'.format(**locals()), lineno=1,
         ))
         finder = make_test_finder(find_links=[data.find_links])
-        resolver = self._basic_resolver(finder)
-        assert_raises_regexp(
-            HashErrors,
-            r'THESE PACKAGES DO NOT MATCH THE HASHES.*\n'
-            r'    file:///.*/data/packages/simple-1\.0\.tar\.gz .*:\n'
-            r'        Expected sha256 badbad\n'
-            r'             Got        393043e672415891885c9a2a0929b1af95fb866d'
-            r'6ca016b42d2e6ce53619b653$',
-            resolver.resolve,
-            reqset)
+        with self._basic_resolver(finder, require_hashes=True) as resolver:
+            assert_raises_regexp(
+                HashErrors,
+                r'THESE PACKAGES DO NOT MATCH THE HASHES.*\n'
+                r'    file:///.*/data/packages/simple-1\.0\.tar\.gz .*:\n'
+                r'        Expected sha256 badbad\n'
+                r'             Got        393043e672415891885c9a2a0929b1af95fb'
+                r'866d6ca016b42d2e6ce53619b653$',
+                resolver.resolve,
+                reqset.all_requirements,
+                True,
+            )
 
     def test_unhashed_deps_on_require_hashes(self, data):
         """Make sure unhashed, unpinned, or otherwise unrepeatable
         dependencies get complained about when --require-hashes is on."""
         reqset = RequirementSet()
         finder = make_test_finder(find_links=[data.find_links])
-        resolver = self._basic_resolver(finder)
         reqset.add_requirement(get_processed_req_from_line(
             'TopoRequires2==0.0.1 '  # requires TopoRequires
             '--hash=sha256:eaf9a01242c9f2f42cf2bd82a6a848cd'
             'e3591d14f7896bdbefcf48543720c970',
             lineno=1
         ))
-        assert_raises_regexp(
-            HashErrors,
-            r'In --require-hashes mode, all requirements must have their '
-            r'versions pinned.*\n'
-            r'    TopoRequires from .*$',
-            resolver.resolve,
-            reqset)
+
+        with self._basic_resolver(finder, require_hashes=True) as resolver:
+            assert_raises_regexp(
+                HashErrors,
+                r'In --require-hashes mode, all requirements must have their '
+                r'versions pinned.*\n'
+                r'    TopoRequires from .*$',
+                resolver.resolve,
+                reqset.all_requirements,
+                True,
+            )
 
     def test_hashed_deps_on_require_hashes(self):
         """Make sure hashed dependencies get installed when --require-hashes
@@ -468,14 +454,14 @@ class TestInstallRequirement(object):
     def test_markers_url(self):
         # test "URL; markers" syntax
         url = 'http://foo.com/?p=bar.git;a=snapshot;h=v0.1;sf=tgz'
-        line = '%s; python_version >= "3"' % url
+        line = '{}; python_version >= "3"'.format(url)
         req = install_req_from_line(line)
         assert req.link.url == url, req.url
         assert str(req.markers) == 'python_version >= "3"'
 
         # without space, markers are part of the URL
         url = 'http://foo.com/?p=bar.git;a=snapshot;h=v0.1;sf=tgz'
-        line = '%s;python_version >= "3"' % url
+        line = '{};python_version >= "3"'.format(url)
         req = install_req_from_line(line)
         assert req.link.url == line, req.url
         assert req.markers is None
@@ -484,7 +470,7 @@ class TestInstallRequirement(object):
         # match
         for markers in (
             'python_version >= "1.0"',
-            'sys_platform == %r' % sys.platform,
+            'sys_platform == {sys.platform!r}'.format(**globals()),
         ):
             line = 'name; ' + markers
             req = install_req_from_line(line)
@@ -494,7 +480,7 @@ class TestInstallRequirement(object):
         # don't match
         for markers in (
             'python_version >= "5.0"',
-            'sys_platform != %r' % sys.platform,
+            'sys_platform != {sys.platform!r}'.format(**globals()),
         ):
             line = 'name; ' + markers
             req = install_req_from_line(line)
@@ -505,7 +491,7 @@ class TestInstallRequirement(object):
         # match
         for markers in (
             'python_version >= "1.0"',
-            'sys_platform == %r' % sys.platform,
+            'sys_platform == {sys.platform!r}'.format(**globals()),
         ):
             line = 'name; ' + markers
             req = install_req_from_line(line, comes_from='')
@@ -515,7 +501,7 @@ class TestInstallRequirement(object):
         # don't match
         for markers in (
             'python_version >= "5.0"',
-            'sys_platform != %r' % sys.platform,
+            'sys_platform != {sys.platform!r}'.format(**globals()),
         ):
             line = 'name; ' + markers
             req = install_req_from_line(line, comes_from='')
@@ -525,7 +511,7 @@ class TestInstallRequirement(object):
     def test_extras_for_line_path_requirement(self):
         line = 'SomeProject[ex1,ex2]'
         filename = 'filename'
-        comes_from = '-r %s (line %s)' % (filename, 1)
+        comes_from = '-r {} (line {})'.format(filename, 1)
         req = install_req_from_line(line, comes_from=comes_from)
         assert len(req.extras) == 2
         assert req.extras == {'ex1', 'ex2'}
@@ -533,7 +519,7 @@ class TestInstallRequirement(object):
     def test_extras_for_line_url_requirement(self):
         line = 'git+https://url#egg=SomeProject[ex1,ex2]'
         filename = 'filename'
-        comes_from = '-r %s (line %s)' % (filename, 1)
+        comes_from = '-r {} (line {})'.format(filename, 1)
         req = install_req_from_line(line, comes_from=comes_from)
         assert len(req.extras) == 2
         assert req.extras == {'ex1', 'ex2'}
@@ -541,7 +527,7 @@ class TestInstallRequirement(object):
     def test_extras_for_editable_path_requirement(self):
         url = '.[ex1,ex2]'
         filename = 'filename'
-        comes_from = '-r %s (line %s)' % (filename, 1)
+        comes_from = '-r {} (line {})'.format(filename, 1)
         req = install_req_from_editable(url, comes_from=comes_from)
         assert len(req.extras) == 2
         assert req.extras == {'ex1', 'ex2'}
@@ -549,7 +535,7 @@ class TestInstallRequirement(object):
     def test_extras_for_editable_url_requirement(self):
         url = 'git+https://url#egg=SomeProject[ex1,ex2]'
         filename = 'filename'
-        comes_from = '-r %s (line %s)' % (filename, 1)
+        comes_from = '-r {} (line {})'.format(filename, 1)
         req = install_req_from_editable(url, comes_from=comes_from)
         assert len(req.extras) == 2
         assert req.extras == {'ex1', 'ex2'}
@@ -584,7 +570,7 @@ class TestInstallRequirement(object):
             install_req_from_line(req_file_path)
         err_msg = e.value.args[0]
         assert "Invalid requirement" in err_msg
-        assert "It looks like a path. It does exist." in err_msg
+        assert "It looks like a path. The path does exist." in err_msg
         assert "appears to be a requirements file." in err_msg
         assert "If that is the case, use the '-r' flag to install" in err_msg
 
@@ -597,10 +583,10 @@ def test_parse_editable_local(
     exists_mock.return_value = isdir_mock.return_value = True
     # mocks needed to support path operations on windows tests
     abspath_mock.return_value = "/some/path"
-    assert parse_editable('.') == (None, 'file:///some/path', None)
+    assert parse_editable('.') == (None, 'file:///some/path', set())
     abspath_mock.return_value = "/some/path/foo"
     assert parse_editable('foo') == (
-        None, 'file:///some/path/foo', None,
+        None, 'file:///some/path/foo', set(),
     )
 
 
@@ -608,7 +594,7 @@ def test_parse_editable_explicit_vcs():
     assert parse_editable('svn+https://foo#egg=foo') == (
         'foo',
         'svn+https://foo#egg=foo',
-        None,
+        set(),
     )
 
 
@@ -616,7 +602,7 @@ def test_parse_editable_vcs_extras():
     assert parse_editable('svn+https://foo#egg=foo[extras]') == (
         'foo[extras]',
         'svn+https://foo#egg=foo[extras]',
-        None,
+        set(),
     )
 
 
@@ -640,10 +626,10 @@ def test_exclusive_environment_markers():
     """Make sure RequirementSet accepts several excluding env markers"""
     eq36 = install_req_from_line(
         "Django>=1.6.10,<1.7 ; python_version == '3.6'")
-    eq36.is_direct = True
+    eq36.user_supplied = True
     ne36 = install_req_from_line(
         "Django>=1.6.10,<1.8 ; python_version != '3.6'")
-    ne36.is_direct = True
+    ne36.user_supplied = True
 
     req_set = RequirementSet()
     req_set.add_requirement(eq36)
@@ -651,17 +637,17 @@ def test_exclusive_environment_markers():
     assert req_set.has_requirement('Django')
 
 
-def test_mismatched_versions(caplog, tmpdir):
-    original_source = os.path.join(DATA_DIR, 'src', 'simplewheel-1.0')
-    source_dir = os.path.join(tmpdir, 'simplewheel')
-    shutil.copytree(original_source, source_dir)
-    req = InstallRequirement(req=Requirement('simplewheel==2.0'),
-                             comes_from=None, source_dir=source_dir)
-    req.prepare_metadata()
+def test_mismatched_versions(caplog):
+    req = InstallRequirement(
+        req=Requirement('simplewheel==2.0'),
+        comes_from=None,
+    )
+    req.source_dir = "/tmp/somewhere"  # make req believe it has been unpacked
+    # Monkeypatch!
+    req._metadata = {"name": "simplewheel", "version": "1.0"}
     req.assert_source_matches_version()
     assert caplog.records[-1].message == (
-        'Requested simplewheel==2.0, '
-        'but installing version 1.0'
+        'Requested simplewheel==2.0, but installing version 1.0'
     )
 
 
